from zur_ecu_client.messages import *
from zur_ecu_client.senml.senml_unit import SenmlUnit
from zur_ecu_client.senml.senml_msg_ecu import Ecu
from zur_ecu_client.senml.senml import Senml
from zur_ecu_client.senml.senml_zur_names import SenmlNames


# Test parsing of all ECU messages
def test_parse_ecu_inverter_info_msg():
    msg = Messages.parse(
        '[{"bn": "ECU", "n": "inverter", "vs": "info"},'
        '{"n": "FLon", "vb": false},'
        '{"n": "FRon", "vb": false},'
        '{"n": "RLon", "vb": false},'
        '{"n": "RRon", "vb": false},'
        '{"n": "FLdcOn", "vb": false},'
        '{"n": "FRdcOn", "vb": false},'
        '{"n": "RLdcOn", "vb": false},'
        '{"n": "RRdcOn", "vb": false},'
        '{"n": "FLready", "vb": false},'
        '{"n": "FRready", "vb": false},'
        '{"n": "RLready", "vb": false},'
        '{"n": "RRready", "vb": false}]'
    )
    assert msg == {
        SenmlNames.ECU_INVERTER_INFO_INVERTERON_FL: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERON_FR: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERON_RL: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERON_RR: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_FL: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_FR: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_RL: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_RR: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_FL: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_FR: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_RL: False,
        SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_RR: False,
    }


def test_parse_ecu_inverter_status_msg():
    msg = Messages.parse(
        '[{"bn": "ECU", "n": "inverter", "vs": "status"},'
        '{"n": "FLderating", "vb": false},'
        '{"n": "FRderating", "vb": false},'
        '{"n": "RLderating", "vb": false},'
        '{"n": "RRderating", "vb": false},'
        '{"n": "FLerror", "vb": false},'
        '{"n": "FRerror", "vb": false},'
        '{"n": "RLerror", "vb": false},'
        '{"n": "RRerror", "vb": false},'
        '{"n": "FLerrorCode", "v": 0},'
        '{"n": "FRerrorCode", "v": 0},'
        '{"n": "RLerrorCode", "v": 0},'
        '{"n": "RRerrorCode", "v": 0}]'
    )
    assert msg == {
        SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_FL: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_FR: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_RL: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_RR: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_FL: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_FR: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_RL: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_RR: False,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_FL: 0,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_FR: 0,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_RL: 0,
        SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_RR: 0,
    }


def test_parse_ecu_inverter_sensor_msg():
    msg = Messages.parse(
        '[{"bn": "ECU", "n": "inverter", "vs": "sensor"},'
        '{"n": "FLtorque", "u": "N", "v": 0},'
        '{"n": "FRtorque", "u": "N", "v": 0},'
        '{"n": "RLtorque", "u": "N", "v": 0},'
        '{"n": "RRtorque", "u": "N", "v": 0},'
        '{"n": "FLinverterVoltage", "u": "V", "v": 0},'
        '{"n": "FRinverterVoltage", "u": "V", "v": 0},'
        '{"n": "RLinverterVoltage", "u": "V", "v": 0},'
        '{"n": "RRinverterVoltage", "u": "V", "v": 0},'
        '{"n": "FLinverterTemp", "u": "Cel", "v": 0},'
        '{"n": "FRinverterTemp", "u": "Cel", "v": 0},'
        '{"n": "RLinverterTemp", "u": "Cel", "v": 0},'
        '{"n": "RRinverterTemp", "u": "Cel", "v": 0},'
        '{"n": "FLspeed", "u": "1/min", "v": 0},'
        '{"n": "FRspeed", "u": "1/min", "v": 0},'
        '{"n": "RLspeed", "u": "1/min", "v": 0},'
        '{"n": "RRspeed", "u": "1/min", "v": 0}]'
    )
    assert msg == {
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_FL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_FR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_RL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_RR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_FL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_FR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_RL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_RR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_FL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_FR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_RL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_RR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_FL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_FR: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_RL: 0,
        SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_RR: 0,
    }


def test_parse_ecu_accu_sensor_msg():
    msg = Messages.parse(
        '[{"bn":"ECU","n":"accu","vs":"sensor"},'
        '{"n":"accuPower","u":"A","v":0},'
        '{"n":"averagePower","u":"A","v":0},'
        '{"n":"charge","u":"%","v":0},'
        '{"n":"voltage","u":"V","v":0},'
        '{"n":"current","u":"A","v":0},'
        '{"n":"accuTemperature","u":"Cel","v":0}]'
    )
    assert msg == {
        SenmlNames.ECU_ACCU_SENSOR_ACCUPOWER: 0,
        SenmlNames.ECU_ACCU_SENSOR_AVERAGEPOWER: 0,
        SenmlNames.ECU_ACCU_SENSOR_ACCUSTATEOFCHARGE: 0,
        SenmlNames.ECU_ACCU_SENSOR_HVVOLTAGE: 0,
        SenmlNames.ECU_ACCU_SENSOR_HVCURRENT: 0,
        SenmlNames.ECU_ACCU_SENSOR_ACCUTEMPERATURE: 0,
    }


def test_parse_ecu_error_code_msg():
    msg = Messages.parse(
        '[{"bn":"ECU","n":"error","vs":"code"},{"n":"code","u":"code","v":0}]'
    )
    assert msg == {
        SenmlNames.ECU_ERROR_CODE_CODE: 0,
    }


def test_parse_ecu_driverless_control_msg():
    msg = Messages.parse(
        '[ {"bn":"ECU","n":"driverless","vs":"control"},'
        '{"n":"steeringAngle","u":"Cel","v":0},'
        '{"n":"throttle","u":"%","v":0},'
        '{"n":"brake","u":"%","v":0}]'
    )

    assert msg == {
        SenmlNames.ECU_DRIVERLESS_CONTROL_STEERINGANGLE: 0,
        SenmlNames.ECU_DRIVERLESS_CONTROL_DVTHROTTLE: 0,
        SenmlNames.ECU_DRIVERLESS_CONTROL_DVBRAKE: 0,
    }


def test_parse_ecu_motor_sensor_msg():
    msg = Messages.parse(
        '[{"bn":"ECU","n":"motor","vs":"sensor"},'
        '{"n":"FLmotorTemp","u":"Cel","v":0},'
        '{"n":"FRmotorTemp","u":"Cel","v":0},'
        '{"n":"BLmotorTemp","u":"Cel","v":0},'
        '{"n":"BRmotorTemp","u":"Cel","v":0}]'
    )
    assert msg == {
        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_FL: 0,
        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_FR: 0,
        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_BL: 0,
        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_BR: 0,
    }


def test_parse_ecu_steering_sensor_msg():
    msg = Messages.parse(
        '[{"bn":"ECU","n":"steering","vs":"sensor"},'
        '{"n":"SteeringCurrent","u":"A","v":0},'
        '{"n":"SteeringErrorCode","v":0},'
        '{"n":"SteeringPosition","v":0},'
        '{"n":"SteeringSpeed","u":"m/s","v":0},'
        '{"n":"SteeringTemperature","u":"Cel","v":0}]'
    )
    assert msg == {
        SenmlNames.ECU_STEERING_SENSOR_STEERINGCURRENT: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGERRORCODE: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGPOSITION: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGSPEED: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGTEMPERATURE: 0,
    }


def test_parse_ecu_pedal_sensor_msg():
    msg = Messages.parse(
        '[{"bn":"ECU","n":"pedal","vs":"sensor"},{"n":"FbreakPressure","u":"Pa","v":0},'
        '{"n":"RbreakPressure","u":"Pa","v":0},{"n":"LthrottleMax","u":"%","v":0},'
        '{"n":"LthrottleMin","u":"%","v":0},{"n":"LthrottleValue","u":"%","v":0},'
        '{"n":"RthrottleMax","u":"%","v":0}, {"n":"RthrottleMin","u":"%","v":0}, {"n":"RthrottleValue","u":"%","v":0}]'
    )
    assert msg == {
        SenmlNames.ECU_PEDAL_SENSOR_BREAKPRESSUREFRONT: 0,
        SenmlNames.ECU_PEDAL_SENSOR_BREAKPRESSUREREAR: 0,
        SenmlNames.ECU_PEDAL_SENSOR_THROTTLELEFTMAX: 0,
        SenmlNames.ECU_PEDAL_SENSOR_THROTTLELEFTMIN: 0,
        SenmlNames.ECU_PEDAL_SENSOR_THROTTLELEFTVALUE: 0,
        SenmlNames.ECU_PEDAL_SENSOR_THROTTLERIGHTMAX: 0,
        SenmlNames.ECU_PEDAL_SENSOR_THROTTLERIGHTMIN: 0,
        SenmlNames.ECU_PEDAL_SENSOR_THROTTLERIGHTVALUE: 0,
    }


def test_parse_ecu_other_status_msg():
    msg = Messages.parse(
        '[{"bn":"ECU","n":"other","vs":"status"},{"n":"AIRNegative","v":0},'
        '{"n":"AIRPositive","v":0},{"n":"DC_DC_24V","v":0},'
        '{"n":"DVon","v":false},{"n":"LV_Battery_24V","v":0},'
        '{"n":"RTDButton","v":0},{"n":"SDC_Pressed","v":0},'
        '{"n":"TSButton","v":0}, {"n":"current","v":0}, {"n":"current","v":0},'
        '{"n":"input_12V","v":0}, {"n":"prechargeRelay","v":0}]'
    )
    assert msg == {
        SenmlNames.ECU_OTHER_STATUS_AIRPOSITIVE: 0,
        SenmlNames.ECU_OTHER_STATUS_AIRNEGATIVE: 0,
        SenmlNames.ECU_OTHER_STATUS_DC_DC_24V: 0,
        SenmlNames.ECU_OTHER_STATUS_DVON: False,
        SenmlNames.ECU_OTHER_STATUS_LV_BATTERY_24: 0,
        SenmlNames.ECU_OTHER_STATUS_RTDBUTTON: 0,
        SenmlNames.ECU_OTHER_STATUS_SDC_PRESSED: 0,
        SenmlNames.ECU_OTHER_STATUS_TSBUTTON: 0,
        SenmlNames.ECU_OTHER_STATUS_CURRENT: 0,
        SenmlNames.ECU_OTHER_STATUS_INPUT_12V: 0,
        SenmlNames.ECU_OTHER_STATUS_PRECHARGERELAY: 0,
    }


# Test building of all ECU messages
def test_build_ecu_inverter_info_msg():
    msg = Ecu.Inverter.Info(
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        False,
    ).get()

    assert msg == [
        {"bn": "ECU", "n": "inverter", "vs": "info"},
        {"n": "FLon", "vb": False},
        {"n": "FRon", "vb": False},
        {"n": "RLon", "vb": False},
        {"n": "RRon", "vb": False},
        {"n": "FLdcOn", "vb": False},
        {"n": "FRdcOn", "vb": False},
        {"n": "RLdcOn", "vb": False},
        {"n": "RRdcOn", "vb": False},
        {"n": "FLready", "vb": False},
        {"n": "FRready", "vb": False},
        {"n": "RLready", "vb": False},
        {"n": "RRready", "vb": False},
    ]


def test_build_ecu_inverter_status_msg():
    msg = Ecu.Inverter.Status(
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        False,
        0,
        0,
        0,
        0,
    ).get()

    assert msg == [
        {"bn": "ECU", "n": "inverter", "vs": "status"},
        {"n": "FLderating", "vb": False},
        {"n": "FRderating", "vb": False},
        {"n": "RLderating", "vb": False},
        {"n": "RRderating", "vb": False},
        {"n": "FLerror", "vb": False},
        {"n": "FRerror", "vb": False},
        {"n": "RLerror", "vb": False},
        {"n": "RRerror", "vb": False},
        {"n": "FLerrorCode", "v": 0},
        {"n": "FRerrorCode", "v": 0},
        {"n": "RLerrorCode", "v": 0},
        {"n": "RRerrorCode", "v": 0},
    ]


def test_build_ecu_inverter_sensor_msg():
    msg = Ecu.Inverter.Sensor(
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
    ).get()

    assert msg == [
        {"bn": "ECU", "n": "inverter", "vs": "sensor"},
        {"n": "FLtorque", "u": "N", "v": 0},
        {"n": "FRtorque", "u": "N", "v": 0},
        {"n": "RLtorque", "u": "N", "v": 0},
        {"n": "RRtorque", "u": "N", "v": 0},
        {"n": "FLinverterVoltage", "u": "V", "v": 0},
        {"n": "FRinverterVoltage", "u": "V", "v": 0},
        {"n": "RLinverterVoltage", "u": "V", "v": 0},
        {"n": "RRinverterVoltage", "u": "V", "v": 0},
        {"n": "FLinverterTemp", "u": "Cel", "v": 0},
        {"n": "FRinverterTemp", "u": "Cel", "v": 0},
        {"n": "RLinverterTemp", "u": "Cel", "v": 0},
        {"n": "RRinverterTemp", "u": "Cel", "v": 0},
        {"n": "FLspeed", "u": "1/min", "v": 0},
        {"n": "FRspeed", "u": "1/min", "v": 0},
        {"n": "RLspeed", "u": "1/min", "v": 0},
        {"n": "RRspeed", "u": "1/min", "v": 0},
    ]


def test_build_ecu_accu_sensor_msg():
    msg = Ecu.Accu(0, 0, 0, 0, 0, 0).get()
    assert msg == [
        {"bn": "ECU", "n": "accu", "vs": "sensor"},
        {"n": "accuPower", "u": "A", "v": 0},
        {"n": "averagePower", "u": "A", "v": 0},
        {"n": "charge", "u": "%", "v": 0},
        {"n": "voltage", "u": "V", "v": 0},
        {"n": "current", "u": "A", "v": 0},
        {"n": "accuTemperature", "u": "Cel", "v": 0},
    ]


def test_build_ecu_error_code_msg():
    msg = Ecu.Error(0).get()
    assert msg == [{"bn": "ECU", "n": "error", "vs": "code"}, {"n": "code", "v": 0}]


def test_build_ecu_driverless_control_msg():
    msg = Ecu.Driverless(0, 0, 0).get()
    assert msg == [
        {"bn": "ECU", "n": "driverless", "vs": "control"},
        {"n": "steeringAngle", "u": "Cel", "v": 0},
        {"n": "throttle", "u": "%", "v": 0},
        {"n": "brake", "u": "%", "v": 0},
    ]


def test_build_ecu_motor_sensor_msg():
    msg = Ecu.Motor(0, 0, 0, 0).get()
    assert msg == [
        {"bn": "ECU", "n": "motor", "vs": "sensor"},
        {"n": "FLmotorTemp", "u": "Cel", "v": 0},
        {"n": "FRmotorTemp", "u": "Cel", "v": 0},
        {"n": "BLmotorTemp", "u": "Cel", "v": 0},
        {"n": "BRmotorTemp", "u": "Cel", "v": 0},
    ]


def test_build_ecu_steering_sensor_msg():
    msg = Ecu.Steering(0, 0, 0, 0, 0).get()
    assert msg == [
        {"bn": "ECU", "n": "steering", "vs": "sensor"},
        {"n": "SteeringCurrent", "u": "A", "v": 0},
        {"n": "SteeringErrorCode", "v": 0},
        {"n": "SteeringPosition", "v": 0},
        {"n": "SteeringSpeed", "u": "m/s", "v": 0},
        {"n": "SteeringTemperature", "u": "Cel", "v": 0},
    ]


def test_build_ecu_pedal_sensor_msg():
    msg = Ecu.Pedal(0, 0, 0, 0, 0, 0, 0, 0).get()
    assert msg == [
        {"bn": "ECU", "n": "pedal", "vs": "sensor"},
        {"n": "FbreakPressure", "u": "Pa", "v": 0},
        {"n": "RbreakPressure", "u": "Pa", "v": 0},
        {"n": "LthrottleMax", "u": "%", "v": 0},
        {"n": "LthrottleMin", "u": "%", "v": 0},
        {"n": "LthrottleValue", "u": "%", "v": 0},
        {"n": "RthrottleMax", "u": "%", "v": 0},
        {"n": "RthrottleMin", "u": "%", "v": 0},
        {"n": "RthrottleValue", "u": "%", "v": 0},
    ]


def test_build_ecu_other_status_msg():
    msg = Ecu.Other(0, 0, 0, False, 0, 0, 0, 0, 0, 0, 0).get()
    assert msg == [
        {"bn": "ECU", "n": "other", "vs": "status"},
        {"n": "AIRNegative", "v": 0},
        {"n": "AIRPositive", "v": 0},
        {"n": "DC_DC_24V", "v": 0},
        {"n": "DVon", "vb": False},
        {"n": "LV_Battery_24V", "v": 0},
        {"n": "RTDButton", "v": 0},
        {"n": "SDC_Pressed", "v": 0},
        {"n": "TSButton", "v": 0},
        {"n": "current", "v": 0},
        {"n": "input_12V", "v": 0},
        {"n": "prechargeRelay", "v": 0},
    ]



from dataclasses import dataclass
import json
from enum import Enum, IntEnum
from typing import List, Optional, Union

from zur_ecu_client.senml.senml_zur_names import SenmlNames
from zur_ecu_client.senml.senml import Senml
from zur_ecu_client.senml.util import value_by_key_prefix


class Messages:
    class AmiState(Enum):
        ACCELERATION = "acceleration"
        SKIDPAD = "skidpad"
        TRACKDRIVE = "trackdrive"
        BRAKETEST = "braketest"
        INSPECTION = "inspection"
        AUTOCROSS = "autocross"

    class EbsState(IntEnum):
        UNAVAILABLE = 1
        ARMED = 2
        ACTIVATED = 3

    class AsState(IntEnum):
        OFF = 1
        READY = 2
        DRIVING = 3
        EMERGENCY_BREAK = 4
        FINISH = 5

    @staticmethod
    def get_from_entry(data: dict):
        if "bn" in data:
            return Senml.Base.from_json(data)
        elif "n" in data:
            return Senml.Record.from_json(data)
        return TypeError

    @classmethod
    def parse(cls, data_str: str) -> dict:
        """
        Parses a senml content.
        Both a list with dictionary, or a list of list with dict are supported.
        e.g. both  `[{"bn": "DV", "n": "ctrl", "v": "sensor"}]` and
        `[[{"bn": "DV", "n": "ctrl", "v": "sensor"}], [{"bn": "DV", "n": "ctrl", "v": "other_value"}]]
        are supported.

        This was implemented this way, since the it was unclear which format the ECU used, or if both formats where used
        """
        data: List = json.loads(data_str)
        if len(data) == 0:
            return {}

        if isinstance(data[0], list):
            converted_data: dict = {}
            for msg in data:
                converted_data.update(cls.__parse_single_list_message(msg))
            return converted_data
        else:
            return cls.__parse_single_list_message(data)

    @staticmethod
    def __parse_single_list_message(data: List) -> dict:
        converted_data = {}
        device: str = ""
        topic: str = ""
        msg_type: str = ""
        header: str = ""
        for entry in data:
            if "bn" in entry:
                device = entry["bn"]
                topic = f':{entry["n"]}' if "n" in entry else ""
                msg_type = (
                    f':{value_by_key_prefix(entry, "v")}'
                    if value_by_key_prefix(entry, "v")
                    else ""
                )
                header = device + topic + msg_type
            else:
                data_name = SenmlNames(header + ":" + entry["n"])
                converted_data[data_name] = value_by_key_prefix(entry, "v")
        return converted_data


class Acknowledgment:
    def __init__(self, base: SenmlNames) -> None:
        self.base = base


class Data:
    def __init__(self, base: SenmlNames, data: List[SenmlNames]) -> None:
        self.base = base
        self.data = data



"""
Example usage of the ECU client.
"""

import logging
import time
from zur_ecu_client.senml.senml_msg_dv import Dv
from zur_ecu_client.senml.senml_zur_names import SenmlNames
from zur_ecu_client.messages import Acknowledgment, Data
from zur_ecu_client.ecu_client import EcuClient


def ecu_message_listener(message: Acknowledgment | Data):
    if isinstance(message, Acknowledgment):
        logging.info(f"Received acknowledgment: {message}")
    elif isinstance(message, Data):
        logging.info(f"Received data: {message}")
    else:
        logging.warning(f"Received unknown message type: {message}")


def main():
    logging.basicConfig(level=logging.INFO)

    # Initialize ECU client
    ecu_ip = "0.0.0.0"
    ecu_port = 2020
    client_ip = "0.0.0.0"
    client_port = 9000
    calls_per_second = 1

    ecu_client = EcuClient(
        ecu_message_listener, ecu_ip, ecu_port, client_ip, client_port, calls_per_second
    )

    ecu_client.start()

    # Subscribe to messages
    ecu_client.subscribe(SenmlNames.ECU_ACCU_ACTUATOR)

    test_message = Dv.Control(1, 1, 1).get()

    try:
        while True:
            ecu_client.send_msg(test_message)
            time.sleep(1)
    except KeyboardInterrupt:
        ecu_client.udpServer.close()
        logging.info("ECU client stopped")


if __name__ == "__main__":
    main()



"""
This example demonstrates how to receive UDP messages using the socket module.
"""

import socket

UDP_IP = "0.0.0.0"
UDP_PORT = 9000

socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
socket.bind((UDP_IP, UDP_PORT))

while True:
    data, addr = socket.recvfrom(1024)
    print(f"Received message: {data} from {addr}")



def value_by_key_prefix(d: dict, partial: str):
    matches = [val for key, val in d.items() if key.startswith(partial)]
    if not matches:
        raise KeyError(partial)
    if len(matches) > 1:
        raise ValueError(f"{partial} matches more than one key")
    return matches[0]



from zur_ecu_client.senml.senml import Senml
from zur_ecu_client.senml.senml_msg import SenmlMessage
from zur_ecu_client.senml.senml_unit import SenmlUnit
from zur_ecu_client.senml.senml_zur_names import SenmlNames


class Ecu:

    class Inverter:
        class Info(SenmlMessage):
            def __init__(
                self,
                FLon: bool = None,
                FRon: bool = None,
                RLon: bool = None,
                RRon: bool = None,
                FLdcOn: bool = None,
                FRdcOn: bool = None,
                RLdcOn: bool = None,
                RRdcOn: bool = None,
                FLready: bool = None,
                FRready: bool = None,
                RLready: bool = None,
                RRready: bool = None,
            ) -> None:
                super().__init__(
                    SenmlNames.ECU_INVERTER_INFO,
                    [
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERON_FL,
                            v=FLon,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERON_FR,
                            v=FRon,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERON_RL,
                            v=RLon,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERON_RR,
                            v=RRon,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_FL,
                            v=FLdcOn,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_FR,
                            v=FRdcOn,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_RL,
                            v=RLdcOn,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERDCON_RR,
                            v=RRdcOn,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_FL,
                            v=FLready,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_FR,
                            v=FRready,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_RL,
                            v=RLready,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_INFO_INVERTERSYSTEMREADY_RR,
                            v=RRready,
                        ),
                    ],
                )

        class Status(SenmlMessage):
            def __init__(
                self,
                FLderating: bool = None,
                FRderating: bool = None,
                RLderating: bool = None,
                RRderating: bool = None,
                FLerror: bool = None,
                FRerror: bool = None,
                RLerror: bool = None,
                RRerror: bool = None,
                FLerrorCode: int = None,
                FRerrorCode: int = None,
                RLerrorCode: int = None,
                RRerrorCode: int = None,
            ) -> None:
                super().__init__(
                    SenmlNames.ECU_INVERTER_STATUS,
                    [
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_FL,
                            v=FLderating,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_FR,
                            v=FRderating,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_RL,
                            v=RLderating,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERDERATNG_RR,
                            v=RRderating,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_FL, v=FLerror
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_FR, v=FRerror
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_RL, v=RLerror
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERROR_RR, v=RRerror
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_FL,
                            v=FLerrorCode,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_FR,
                            v=FRerrorCode,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_RL,
                            v=RLerrorCode,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_STATUS_INVERTERERRORCODE_RR,
                            v=RRerrorCode,
                        ),
                    ],
                )

        class Sensor(SenmlMessage):
            def __init__(
                self,
                FLtorque: int = None,
                FRtorque: int = None,
                RLtorque: int = None,
                RRtorque: int = None,
                FLinverterVoltage: int = None,
                FRinverterVoltage: int = None,
                RLinverterVoltage: int = None,
                RRinverterVoltage: int = None,
                FLinverterTemp: int = None,
                FRinverterTemp: int = None,
                RLinverterTemp: int = None,
                RRinverterTemp: int = None,
                FLspeed: int = None,
                FRspeed: int = None,
                RLspeed: int = None,
                RRspeed: int = None,
            ) -> None:
                super().__init__(
                    SenmlNames.ECU_INVERTER_SENSOR,
                    [
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_FL,
                            SenmlUnit.NEWTON,
                            v=FLtorque,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_FR,
                            SenmlUnit.NEWTON,
                            v=FRtorque,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_RL,
                            SenmlUnit.NEWTON,
                            v=RLtorque,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_RR,
                            SenmlUnit.NEWTON,
                            v=RRtorque,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_FL,
                            SenmlUnit.VOLT,
                            v=FLinverterVoltage,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_FR,
                            SenmlUnit.VOLT,
                            v=FRinverterVoltage,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_RL,
                            SenmlUnit.VOLT,
                            v=RLinverterVoltage,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_RR,
                            SenmlUnit.VOLT,
                            v=RRinverterVoltage,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_FL,
                            SenmlUnit.DEGREES_CELSIUS,
                            v=FLinverterTemp,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_FR,
                            SenmlUnit.DEGREES_CELSIUS,
                            v=FRinverterTemp,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_RL,
                            SenmlUnit.DEGREES_CELSIUS,
                            v=RLinverterTemp,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERTEMP_RR,
                            SenmlUnit.DEGREES_CELSIUS,
                            v=RRinverterTemp,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_FL,
                            SenmlUnit.EVENT_RATE_PER_MINUTE,
                            v=FLspeed,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_FR,
                            SenmlUnit.EVENT_RATE_PER_MINUTE,
                            v=FRspeed,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_RL,
                            SenmlUnit.EVENT_RATE_PER_MINUTE,
                            v=RLspeed,
                        ),
                        Senml.Record(
                            SenmlNames.ECU_INVERTER_SENSOR_INVERTERACTUALRPM_RR,
                            SenmlUnit.EVENT_RATE_PER_MINUTE,
                            v=RRspeed,
                        ),
                    ],
                )

    class Accu(SenmlMessage):
        def __init__(
            self,
            accuPower: int = None,
            averagePower: int = None,
            charge: int = None,
            voltage: int = None,
            current: int = None,
            accuTemperature: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_ACCU_SENSOR,
                [
                    Senml.Record(
                        SenmlNames.ECU_ACCU_SENSOR_ACCUPOWER,
                        SenmlUnit.AMPERE,
                        accuPower,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_ACCU_SENSOR_AVERAGEPOWER,
                        SenmlUnit.AMPERE,
                        averagePower,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_ACCU_SENSOR_ACCUSTATEOFCHARGE,
                        SenmlUnit.PERCENTAGE,
                        charge,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_ACCU_SENSOR_HVVOLTAGE,
                        SenmlUnit.VOLT,
                        voltage,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_ACCU_SENSOR_HVCURRENT,
                        SenmlUnit.AMPERE,
                        current,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_ACCU_SENSOR_ACCUTEMPERATURE,
                        SenmlUnit.DEGREES_CELSIUS,
                        accuTemperature,
                    ),
                ],
            )

    class Error(SenmlMessage):
        def __init__(
            self,
            code: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_ERROR_CODE,
                [
                    Senml.Record(
                        SenmlNames.ECU_ERROR_CODE_CODE,
                        v=code,
                    ),
                ],
            )

    class Driverless(SenmlMessage):
        def __init__(
            self,
            steeringAngle: int = None,
            throttle: int = None,
            brake: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_DRIVERLESS_CONTROL,
                [
                    Senml.Record(
                        SenmlNames.ECU_DRIVERLESS_CONTROL_STEERINGANGLE,
                        SenmlUnit.DEGREES_CELSIUS,
                        steeringAngle,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_DRIVERLESS_CONTROL_DVTHROTTLE,
                        SenmlUnit.PERCENTAGE,
                        throttle,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_DRIVERLESS_CONTROL_DVBRAKE,
                        SenmlUnit.PERCENTAGE,
                        brake,
                    ),
                ],
            )

    class Motor(SenmlMessage):
        def __init__(
            self,
            FLmotorTemp: int = None,
            FRmotorTemp: int = None,
            BLmotorTemp: int = None,
            BRmotorTemp: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_MOTOR_SENSOR,
                [
                    Senml.Record(
                        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_FL,
                        SenmlUnit.DEGREES_CELSIUS,
                        FLmotorTemp,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_FR,
                        SenmlUnit.DEGREES_CELSIUS,
                        FRmotorTemp,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_BL,
                        SenmlUnit.DEGREES_CELSIUS,
                        BLmotorTemp,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_MOTOR_SENSOR_MOTORTEMP_BR,
                        SenmlUnit.DEGREES_CELSIUS,
                        BRmotorTemp,
                    ),
                ],
            )

    class Steering(SenmlMessage):
        def __init__(
            self,
            SteeringCurrent: int = None,
            SteeringErrorCode: int = None,
            SteeringPosition: int = None,
            SteeringSpeed: int = None,
            SteeringTemperature: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_STEERING_SENSOR,
                [
                    Senml.Record(
                        SenmlNames.ECU_STEERING_SENSOR_STEERINGCURRENT,
                        SenmlUnit.AMPERE,
                        SteeringCurrent,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_STEERING_SENSOR_STEERINGERRORCODE,
                        None,
                        SteeringErrorCode,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_STEERING_SENSOR_STEERINGPOSITION,
                        None,
                        SteeringPosition,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_STEERING_SENSOR_STEERINGSPEED,
                        SenmlUnit.VELOCITY,
                        SteeringSpeed,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_STEERING_SENSOR_STEERINGTEMPERATURE,
                        SenmlUnit.DEGREES_CELSIUS,
                        SteeringTemperature,
                    ),
                ],
            )

    class Pedal(SenmlMessage):
        def __init__(
            self,
            FbreakPressure: int = None,
            RbreakPressure: int = None,
            LthrottleMax: int = None,
            LthrottleMin: int = None,
            LthrottleValue: int = None,
            RthrottleMax: int = None,
            RthrottleMin: int = None,
            RthrottleValue: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_PEDAL_SENSOR,
                [
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_BREAKPRESSUREFRONT,
                        SenmlUnit.PASCAL,
                        FbreakPressure,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_BREAKPRESSUREREAR,
                        SenmlUnit.PASCAL,
                        RbreakPressure,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_THROTTLELEFTMAX,
                        SenmlUnit.PERCENTAGE,
                        LthrottleMax,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_THROTTLELEFTMIN,
                        SenmlUnit.PERCENTAGE,
                        LthrottleMin,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_THROTTLELEFTVALUE,
                        SenmlUnit.PERCENTAGE,
                        LthrottleValue,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_THROTTLERIGHTMAX,
                        SenmlUnit.PERCENTAGE,
                        RthrottleMax,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_THROTTLERIGHTMIN,
                        SenmlUnit.PERCENTAGE,
                        RthrottleMin,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_PEDAL_SENSOR_THROTTLERIGHTVALUE,
                        SenmlUnit.PERCENTAGE,
                        RthrottleValue,
                    ),
                ],
            )

    class Other(SenmlMessage):
        def __init__(
            self,
            AIRNegative: int = None,
            AIRPositive: int = None,
            DC_DC_24V: int = None,
            DVon: bool = None,
            LV_Battery_24V: int = None,
            RTDButton: int = None,
            SDC_Pressed: int = None,
            TSButton: int = None,
            current: int = None,
            input_12V: int = None,
            prechargeRelay: int = None,
        ) -> None:
            super().__init__(
                SenmlNames.ECU_OTHER_STATUS,
                [
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_AIRNEGATIVE,
                        None,
                        AIRNegative,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_AIRPOSITIVE,
                        None,
                        AIRPositive,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_DC_DC_24V,
                        None,
                        DC_DC_24V,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_DVON,
                        False,
                        DVon,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_LV_BATTERY_24,
                        None,
                        LV_Battery_24V,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_RTDBUTTON,
                        None,
                        RTDButton,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_SDC_PRESSED,
                        None,
                        SDC_Pressed,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_TSBUTTON,
                        None,
                        TSButton,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_CURRENT,
                        None,
                        current,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_INPUT_12V,
                        None,
                        input_12V,
                    ),
                    Senml.Record(
                        SenmlNames.ECU_OTHER_STATUS_PRECHARGERELAY,
                        None,
                        prechargeRelay,
                    ),
                ],
            )



import json
import logging
import time
import csv


from zur_ecu_client.messages import Acknowledgment, Data, Messages
from zur_ecu_client.senml.senml import Senml
from zur_ecu_client.udp_server import UdpServer
from zur_ecu_client.senml.senml_msg_ecu import Ecu
from zur_ecu_client.senml.senml_zur_names import *


class MockECU:

    def __init__(self, mock_ecu_ip: str, mock_ecu_port: int, dv_ip: str, dv_port: int):
        logging.basicConfig(level=logging.DEBUG)
        self.mock_ecu_ip = mock_ecu_ip
        self.mock_ecu_port = mock_ecu_port
        self.dv_ip = dv_ip
        self.dv_port = dv_port
        self.udp_server = UdpServer(mock_ecu_ip, mock_ecu_port, dv_ip, dv_port)

    @staticmethod
    def do_every(period, f, *args):
        def g_tick():
            t = time.time()
            while True:
                t += period
                yield max(t - time.time(), 0)

        g = g_tick()
        while True:
            time.sleep(next(g))
            f(*args)

    def response_msg(self, mock_data: list):

        mission_msgs, accu_msgs, velocity_msgs, error_msgs = mock_data
        mission_count = 0

        while True:
            data = self.udp_server.receive_data()
            if data:

                data = Messages.parse2(data)

                for element in data:
                    result = []

                    if mission_count >= len(mission_msgs):
                        return

                    # if element is a Request from Client
                    if type(element) is Acknowledgment:
                        base: Senml.Base = element.base
                        base_name = base.bn + base.n + base.v
                        if base_name == SenmlNames.ECU_ACCU_SENSOR.value:
                            result = accu_msgs[mission_count]
                        elif base_name == SenmlNames.ECU_INVERTER_ACTUAL.value:
                            result = velocity_msgs[mission_count]
                        elif base_name == SenmlNames.ECU_ERROR_INFO.value:
                            result = error_msgs[mission_count]
                        elif base_name == SenmlNames.ECU_TEMPORARY_INFO.value:
                            if mission_count >= len(mission_msgs):
                                return
                            else:
                                result = mission_msgs[mission_count]
                                mission_count += 1
                        else:
                            result = []
                        # result = get_mock_data(mock_data, base.n)

                        self.udp_server.send_data(json.dumps(result))

                    # if element is a Data from Client
                    elif type(element) is Data:

                        base: Senml.Base = element.base
                        send = {
                            "bn": str(base.bn).replace(":", ""),
                            "n": str(base.n).replace(":", ""),
                            "vs": str(base.v).replace(":", ""),
                        }
                        self.udp_server.send_data(str(send))


def __main__():
    client_ip = "127.0.0.1"
    client_port = 9000
    mock_ecu_ip = "127.0.0.1"
    mock_ecu_port = 9001
    mock_ecu = MockECU(mock_ecu_ip, mock_ecu_port, client_ip, client_port)
    try:
        demo_data = [[], [], [], []]
        with open("../Demo.csv", "r") as f:
            reader = csv.reader(f, delimiter=",")
            for i, line in enumerate(reader):
                if i == 0:
                    pass
                if i > 0:
                    demo_data[0].append(
                        Ecu.Temporary(
                            available=line[0] == "TRUE" if line[0] != "" else False,
                            mission_sel=int(line[1]) if line[1] != "" else None,
                            mission_ack=int(line[2]) if line[2] != "" else None,
                            startup=line[3] == "TRUE" if line[3] != "" else False,
                            shutdown=line[4] == "TRUE" if line[4] != "" else False,
                        ).get()
                    )
                    demo_data[1].append(
                        Ecu.Accu(
                            charge=int(line[5]) if line[5] != "" else None,
                            temp=int(line[6]) if line[6] != "" else None,
                        ).get()
                    )
                    demo_data[2].append(
                        Ecu.Inverter(
                            velocity=int(line[7]) if line[7] != "" else None,
                        ).get()
                    )
                    demo_data[3].append(
                        Ecu.Error(
                            error_icon=line[8] if line[8] != "" else None,
                            error_message=line[9] if line[9] != "" else None,
                        ).get()
                    )
        mock_ecu.response_msg(demo_data)
        # mock_ecu.do_every(1, mock_ecu.response_msg, msgs)
    except KeyboardInterrupt:
        mock_ecu.udp_server.close()


if __name__ == "__main__":
    __main__()



import logging
import socket


class UdpServer:

    def __init__(self, udpIp: str, udpPort: int, remoteIp: str, remotePort: int):
        self.udpPort = udpPort
        self.udpIp = udpIp
        self.remotePort = remotePort
        self.remoteIp = remoteIp
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.bind((udpIp, udpPort))
        logging.info("Server is running on {}:{}".format(self.udpIp, self.udpPort))

    def receive_data(self):
        data, addr = self.socket.recvfrom(1024)
        data = data.decode("utf-8")
        logging.debug("Received from {}:{} -> {}".format(addr[0], addr[1], data))
        return data

    def send_data(self, data) -> None:
        data = data.encode("utf-8")
        self.socket.sendto(data, (self.remoteIp, self.remotePort))
        logging.debug(
            "Sent to {}:{} -> {}".format(self.remoteIp, self.remotePort, data)
        )

    def close(self):
        self.socket.close()



"""
This example demonstrates how to send and receive SenML message from the ECU using UDP.
"""

import socket
import json

# Set the IP and port of the ECU
ECU_IP = "0.0.0.0"
ECU_PORT = 9000

# SenML message to be send
data = [
    [
        {"bn": "ECU", "n": "driverless", "vs": "control"},
        {"n": "steeringAngle", "u": "°", "v": -2000},
    ]
]

MESSAGE = json.dumps(data)
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

# Send message
sock.sendto(MESSAGE.encode(), (ECU_IP, ECU_PORT))
print(f"Send msg: {MESSAGE}")

# Receive message
response, addr = sock.recvfrom(1024)
response = response.decode()
print(f"Received msg: {response}")



"""
This is a simple example of how to send a message to the ECU using the UDP protocol.
After each sent message the script waits for a response from the ECU.
"""

import socket
import time


# Set the IP and port of the ECU
UDP_IP = "192.168.3.171"
UDP_PORT = 2020

data = b'[[{"bn":"ECU","n":"inverter","vs":"info"}]]'

socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
while True:
    socket.sendto(data, (UDP_IP, UDP_PORT))
    print(f"Send message: {data} to {UDP_IP, UDP_PORT}")
    print(socket.recvfrom(4096))
    time.sleep(1)



"""
This example demonstrates how to send a UDP message to the ECU.
"""

import socket
import time

UDP_IP = "0.0.0.0"
UDP_PORT = 2020

data = b"test"

socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
while True:
    socket.sendto(data, (UDP_IP, UDP_PORT))
    print(f"Send message: {data} to {UDP_IP, UDP_PORT}")
    time.sleep(1)



import json
import logging
import sched
import time
from threading import Thread
from typing import Callable, Union

from zur_ecu_client.senml.senml_msg_dv import SenmlNames, Dv
from zur_ecu_client.udp_server import UdpServer
from zur_ecu_client.messages import Messages, Acknowledgment, Data

EcuClientListener = Callable[[Acknowledgment | Data], None]


class EcuClient:

    def __init__(
        self,
        listener: EcuClientListener,
        ecu_ip: str,
        ecu_port: int,
        client_ip: str = "0.0.0.0",
        client_port: int = 9000,
        calls_per_second: int = 1,
    ) -> None:
        logging.basicConfig(level=logging.CRITICAL)
        self.listener = listener
        self.requestInterval = 1.0 / calls_per_second
        self.subscriptions: list[SenmlNames] = []
        self.compiledMessages = []

        self.client_ip = client_ip
        self.client_port = client_port

        self.udpServer = UdpServer(self.client_ip, self.client_port, ecu_ip, ecu_port)

        self.thread1 = Thread(target=self.__receive_msg)
        # self.thread1.daemon = True
        self.thread2 = Thread(target=self.__schedule_requests)
        # self.thread2.daemon = True

    def start(self):
        self.thread1.start()
        self.thread2.start()

    def subscribe(self, data_field: Union[SenmlNames, str]):
        if type(data_field) is not SenmlNames and SenmlNames(data_field):
            data_field = SenmlNames(data_field)
        if data_field not in self.subscriptions:
            self.subscriptions.append(data_field)
        self.__compile_subscriptions()

    def unsubscribe(self, data_field: Union[SenmlNames, str]):
        if data_field in self.subscriptions:
            self.subscriptions.remove(data_field)
            self.__compile_subscriptions()

    def unsubscribe_all(self):
        self.subscriptions = []
        self.__compile_subscriptions()

    def send_msg(self, msg):
        if not msg:
            return
        msg = json.dumps(msg)
        self.udpServer.send_data(msg)

    def __compile_subscriptions(self):
        self.compiledMessages = []
        for entry in self.subscriptions:
            parameters = entry.value.split(":")
            compiled = [{"bn": parameters[0], "n": parameters[1], "vs": parameters[2]}]
            if compiled not in self.compiledMessages:
                self.compiledMessages.append(
                    [{"bn": parameters[0], "n": parameters[1], "vs": parameters[2]}]
                )

    def __receive_msg(self):
        while True:
            data = self.udpServer.receive_data()
            if data:
                try:
                    messages = Messages.parse(data)
                    self.listener(messages)
                    logging.info(f"Received -> {messages}")
                except Exception as error:
                    logging.error(f"{error} -> Could not parse message: {data}")

    def __request_messages(self):
        self.send_msg(self.compiledMessages)

    def __schedule_requests(self):
        scheduler = sched.scheduler(time.time, time.sleep)
        while True:
            scheduler.enter(self.requestInterval, 1, self.__request_messages, ())
            scheduler.run()



from enum import Enum


class SenmlNames(Enum):
    # ========== ECU ==========
    # ECU:inverter:info
    ECU_INVERTER_INFO = "ECU:inverter:info"
    ECU_INVERTER_INFO_INVERTERON_FL = "ECU:inverter:info:FLon"
    ECU_INVERTER_INFO_INVERTERON_FR = "ECU:inverter:info:FRon"
    ECU_INVERTER_INFO_INVERTERON_RL = "ECU:inverter:info:RLon"
    ECU_INVERTER_INFO_INVERTERON_RR = "ECU:inverter:info:RRon"
    ECU_INVERTER_INFO_INVERTERDCON_FL = "ECU:inverter:info:FLdcOn"
    ECU_INVERTER_INFO_INVERTERDCON_FR = "ECU:inverter:info:FRdcOn"
    ECU_INVERTER_INFO_INVERTERDCON_RL = "ECU:inverter:info:RLdcOn"
    ECU_INVERTER_INFO_INVERTERDCON_RR = "ECU:inverter:info:RRdcOn"
    ECU_INVERTER_INFO_INVERTERSYSTEMREADY_FL = "ECU:inverter:info:FLready"
    ECU_INVERTER_INFO_INVERTERSYSTEMREADY_FR = "ECU:inverter:info:FRready"
    ECU_INVERTER_INFO_INVERTERSYSTEMREADY_RL = "ECU:inverter:info:RLready"
    ECU_INVERTER_INFO_INVERTERSYSTEMREADY_RR = "ECU:inverter:info:RRready"
    # ECU:inverter:status
    ECU_INVERTER_STATUS = "ECU:inverter:status"
    ECU_INVERTER_STATUS_INVERTERDERATNG_FL = "ECU:inverter:status:FLderating"
    ECU_INVERTER_STATUS_INVERTERDERATNG_FR = "ECU:inverter:status:FRderating"
    ECU_INVERTER_STATUS_INVERTERDERATNG_RL = "ECU:inverter:status:RLderating"
    ECU_INVERTER_STATUS_INVERTERDERATNG_RR = "ECU:inverter:status:RRderating"
    ECU_INVERTER_STATUS_INVERTERERROR_FL = "ECU:inverter:status:FLerror"
    ECU_INVERTER_STATUS_INVERTERERROR_FR = "ECU:inverter:status:FRerror"
    ECU_INVERTER_STATUS_INVERTERERROR_RL = "ECU:inverter:status:RLerror"
    ECU_INVERTER_STATUS_INVERTERERROR_RR = "ECU:inverter:status:RRerror"
    ECU_INVERTER_STATUS_INVERTERERRORCODE_FL = "ECU:inverter:status:FLerrorCode"
    ECU_INVERTER_STATUS_INVERTERERRORCODE_FR = "ECU:inverter:status:FRerrorCode"
    ECU_INVERTER_STATUS_INVERTERERRORCODE_RL = "ECU:inverter:status:RLerrorCode"
    ECU_INVERTER_STATUS_INVERTERERRORCODE_RR = "ECU:inverter:status:RRerrorCode"
    # ECU:inverter:sensor
    ECU_INVERTER_SENSOR = "ECU:inverter:sensor"
    ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_FL = "ECU:inverter:sensor:FLtorque"
    ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_FR = "ECU:inverter:sensor:FRtorque"
    ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_RL = "ECU:inverter:sensor:RLtorque"
    ECU_INVERTER_SENSOR_INVERTERACTUALTORQUE_RR = "ECU:inverter:sensor:RRtorque"
    ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_FL = "ECU:inverter:sensor:FLinverterVoltage"
    ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_FR = "ECU:inverter:sensor:FRinverterVoltage"
    ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_RL = "ECU:inverter:sensor:RLinverterVoltage"
    ECU_INVERTER_SENSOR_INVERTERSVOLTAGE_RR = "ECU:inverter:sensor:RRinverterVoltage"
    ECU_INVERTER_SENSOR_INVERTERTEMP_FL = "ECU:inverter:sensor:FLinverterTemp"
    ECU_INVERTER_SENSOR_INVERTERTEMP_FR = "ECU:inverter:sensor:FRinverterTemp"
    ECU_INVERTER_SENSOR_INVERTERTEMP_RL = "ECU:inverter:sensor:RLinverterTemp"
    ECU_INVERTER_SENSOR_INVERTERTEMP_RR = "ECU:inverter:sensor:RRinverterTemp"
    ECU_INVERTER_SENSOR_INVERTERACTUALRPM_FL = "ECU:inverter:sensor:FLspeed"
    ECU_INVERTER_SENSOR_INVERTERACTUALRPM_FR = "ECU:inverter:sensor:FRspeed"
    ECU_INVERTER_SENSOR_INVERTERACTUALRPM_RL = "ECU:inverter:sensor:RLspeed"
    ECU_INVERTER_SENSOR_INVERTERACTUALRPM_RR = "ECU:inverter:sensor:RRspeed"
    # ECU:accu:sensor
    ECU_ACCU_SENSOR = "ECU:accu:sensor"
    ECU_ACCU_SENSOR_ACCUPOWER = "ECU:accu:sensor:accuPower"
    ECU_ACCU_SENSOR_AVERAGEPOWER = "ECU:accu:sensor:averagePower"
    ECU_ACCU_SENSOR_ACCUSTATEOFCHARGE = "ECU:accu:sensor:charge"
    ECU_ACCU_SENSOR_HVVOLTAGE = "ECU:accu:sensor:voltage"
    ECU_ACCU_SENSOR_HVCURRENT = "ECU:accu:sensor:current"
    ECU_ACCU_SENSOR_ACCUTEMPERATURE = "ECU:accu:sensor:accuTemperature"
    # ECU:error:code
    ECU_ERROR_CODE = "ECU:error:code"
    ECU_ERROR_CODE_CODE = "ECU:error:code:code"
    # ECU:driverless:control
    ECU_DRIVERLESS_CONTROL = "ECU:driverless:control"
    ECU_DRIVERLESS_CONTROL_STEERINGANGLE = "ECU:driverless:control:steeringAngle"
    ECU_DRIVERLESS_CONTROL_DVTHROTTLE = "ECU:driverless:control:throttle"
    ECU_DRIVERLESS_CONTROL_DVBRAKE = "ECU:driverless:control:brake"
    # ECU:motor:sensor
    ECU_MOTOR_SENSOR = "ECU:motor:sensor"
    ECU_MOTOR_SENSOR_MOTORTEMP_FL = "ECU:motor:sensor:FLmotorTemp"
    ECU_MOTOR_SENSOR_MOTORTEMP_FR = "ECU:motor:sensor:FRmotorTemp"
    ECU_MOTOR_SENSOR_MOTORTEMP_BL = "ECU:motor:sensor:BLmotorTemp"
    ECU_MOTOR_SENSOR_MOTORTEMP_BR = "ECU:motor:sensor:BRmotorTemp"
    # ECU:steering:sensor
    ECU_STEERING_SENSOR = "ECU:steering:sensor"
    ECU_STEERING_SENSOR_STEERINGCURRENT = "ECU:steering:sensor:SteeringCurrent"
    ECU_STEERING_SENSOR_STEERINGERRORCODE = "ECU:steering:sensor:SteeringErrorCode"
    ECU_STEERING_SENSOR_STEERINGPOSITION = "ECU:steering:sensor:SteeringPosition"
    ECU_STEERING_SENSOR_STEERINGSPEED = "ECU:steering:sensor:SteeringSpeed"
    ECU_STEERING_SENSOR_STEERINGTEMPERATURE = "ECU:steering:sensor:SteeringTemperature"
    # ECU:pedal:sensor
    ECU_PEDAL_SENSOR = "ECU:pedal:sensor"
    ECU_PEDAL_SENSOR_BREAKPRESSUREFRONT = "ECU:pedal:sensor:FbreakPressure"
    ECU_PEDAL_SENSOR_BREAKPRESSUREREAR = "ECU:pedal:sensor:RbreakPressure"
    ECU_PEDAL_SENSOR_THROTTLELEFTMAX = "ECU:pedal:sensor:LthrottleMax"
    ECU_PEDAL_SENSOR_THROTTLELEFTMIN = "ECU:pedal:sensor:LthrottleMin"
    ECU_PEDAL_SENSOR_THROTTLELEFTVALUE = "ECU:pedal:sensor:LthrottleValue"
    ECU_PEDAL_SENSOR_THROTTLERIGHTMAX = "ECU:pedal:sensor:RthrottleMax"
    ECU_PEDAL_SENSOR_THROTTLERIGHTMIN = "ECU:pedal:sensor:RthrottleMin"
    ECU_PEDAL_SENSOR_THROTTLERIGHTVALUE = "ECU:pedal:sensor:RthrottleValue"
    # ECU:other:status
    ECU_OTHER_STATUS = "ECU:other:status"
    ECU_OTHER_STATUS_AIRNEGATIVE = "ECU:other:status:AIRNegative"
    ECU_OTHER_STATUS_AIRPOSITIVE = "ECU:other:status:AIRPositive"
    ECU_OTHER_STATUS_DC_DC_24V = "ECU:other:status:DC_DC_24V"
    ECU_OTHER_STATUS_DVON = "ECU:other:status:DVon"
    ECU_OTHER_STATUS_LV_BATTERY_24 = "ECU:other:status:LV_Battery_24V"
    ECU_OTHER_STATUS_RTDBUTTON = "ECU:other:status:RTDButton"
    ECU_OTHER_STATUS_SDC_PRESSED = "ECU:other:status:SDC_Pressed"
    ECU_OTHER_STATUS_TSBUTTON = "ECU:other:status:TSButton"
    ECU_OTHER_STATUS_CURRENT = "ECU:other:status:current"
    ECU_OTHER_STATUS_INPUT_12V = "ECU:other:status:input_12V"
    ECU_OTHER_STATUS_PRECHARGERELAY = "ECU:other:status:prechargeRelay"



from zur_ecu_client.messages import Acknowledgment, Data, Messages
from zur_ecu_client.senml.senml import Senml
from zur_ecu_client.senml.senml_zur_names import SenmlNames


def test_incoming_parser():
    msg = Messages.parse(
        """
        [
            [
                {"bn":"ECU","n":"pedal","vs":"sensor"}
            ],
            [
                {"bn":"ECU","n":"other","vs":"status"},
                {"n":"AIRPositive","u":"","v":0},
                {"n":"AIRNegative","u":"","v":0},
                {"n":"DC_DC_24V","u":"","v":0},
                {"n":"DVon","vb":false},
                {"n":"LV_Battery_24V","u":"","v":0},
                {"n":"RTDButton","u":"","v":0},
                {"n":"SDC_Pressed","u":"","v":0},
                {"n":"TSButton","u":"","v":0}, 
                {"n":"current","u":"","v":0}, 
                {"n":"input_12V","u":"","v":0}, 
                {"n":"prechargeRelay","u":"","v":0}
            ]
        ]"""
    )

    assert msg == {
        SenmlNames.ECU_OTHER_STATUS_AIRPOSITIVE: 0,
        SenmlNames.ECU_OTHER_STATUS_AIRNEGATIVE: 0,
        SenmlNames.ECU_OTHER_STATUS_DC_DC_24V: 0,
        SenmlNames.ECU_OTHER_STATUS_DVON: False,
        SenmlNames.ECU_OTHER_STATUS_LV_BATTERY_24: 0,
        SenmlNames.ECU_OTHER_STATUS_RTDBUTTON: 0,
        SenmlNames.ECU_OTHER_STATUS_SDC_PRESSED: 0,
        SenmlNames.ECU_OTHER_STATUS_TSBUTTON: 0,
        SenmlNames.ECU_OTHER_STATUS_CURRENT: 0,
        SenmlNames.ECU_OTHER_STATUS_INPUT_12V: 0,
        SenmlNames.ECU_OTHER_STATUS_PRECHARGERELAY: 0,
    }


def test_incoming_parser_single_list_message():
    msg = Messages.parse(
        """
            [
                {"bn":"ECU","n":"steering","vs":"sensor"},
                {"n":"SteeringCurrent","u":"A","v":0},
                {"n":"SteeringErrorCode","v":0},
                {"n":"SteeringPosition","u":"lat","v":0},
                {"n":"SteeringSpeed","u":"m/s","v":0}, 
                {"n":"SteeringTemperature","u":"Cel","v":0}
            ]
        """
    )

    assert msg == {
        SenmlNames.ECU_STEERING_SENSOR_STEERINGCURRENT: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGERRORCODE: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGPOSITION: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGSPEED: 0,
        SenmlNames.ECU_STEERING_SENSOR_STEERINGTEMPERATURE: 0,
    }



from typing import List

from zur_ecu_client.senml.senml import Senml
from zur_ecu_client.senml.senml_unit import SenmlUnit
from zur_ecu_client.senml.senml_zur_names import SenmlNames


class SenmlMessage:
    def __init__(self, msg_type: SenmlNames, msg_map: List[Senml.Record]) -> None:
        msg_type: List = msg_type.value.split(":")
        self.baseName = msg_type[0]
        self.name = msg_type[1]
        self.value = msg_type[2] if len(msg_type) > 2 else ""
        self.map = msg_map

    def get(self):
        msg = []
        msg_base = {"bn": self.baseName, "n": self.name}
        if self.value:
            msg_base["vs"] = self.value
        msg.append(msg_base)
        for item in self.map:
            if item.v is None:
                continue
            key = item.n.value.split(":")[-1]
            if type(item.v) is str:
                msg.append({"n": key, "vs": item.v})
            elif type(item.v) is bool:
                msg.append({"n": key, "vb": item.v})
            elif type(item.u) is SenmlUnit:
                msg.append({"n": key, "u": item.u.value, "v": item.v})
            else:
                msg.append({"n": key, "v": item.v})
        return msg



from enum import Enum

from zur_ecu_client.senml.senml_unit import SenmlUnit
from zur_ecu_client.senml.senml_zur_names import SenmlNames
from zur_ecu_client.senml.util import value_by_key_prefix


class Senml:
    class Type(Enum):
        REQUEST = 1
        DATA = 2
        ACKNOWLEDGMENT = 3

    class Base:
        def __init__(self, bn, n, v):
            self.bn = bn
            self.n = n
            self.v = v

        @classmethod
        def from_json(cls, data):
            device = data["bn"]
            topic = f':{data["n"]}' if "n" in data else ""
            msg_type = (
                f':{value_by_key_prefix(data, "v")}'
                if value_by_key_prefix(data, "v")
                else ""
            )
            return cls(device, topic, msg_type)

    class Record:
        def __init__(self, n: SenmlNames, u: SenmlUnit = None, v: any = None):
            self.n = n
            self.u = u
            self.v = v

        @classmethod
        def from_json(cls, data):
            device = data["n"]
            unit = f':{data["u"]}' if "u" in data else None
            value = (
                f':{value_by_key_prefix(data, "v")}'
                if value_by_key_prefix(data, "v")
                else ""
            )
            return cls(device, unit, value)

    class Pack:
        def __init__(self, base, records: list = None):
            self.records = []
            self.base = base
            if records:
                self.records.append(records)



from enum import Enum


class SenmlUnit(Enum):
    METER = "m"
    KILOGRAM = "kg"
    GRAM = "g"
    SECOND = "s"
    AMPERE = "A"
    KELVIN = "K"
    CANDELA = "cd"
    MOLE = "mol"
    HERTZ = "Hz"
    RADIAN = "rad"
    STERADIAN = "sr"
    NEWTON = "N"
    PASCAL = "Pa"
    JOULE = "J"
    WATT = "W"
    COULOMB = "C"
    VOLT = "V"
    FARAD = "F"
    OHM = "Ohm"
    SIEMENS = "S"
    WEBER = "Wb"
    TESLA = "T"
    HENRY = "H"
    DEGREES_CELSIUS = "Cel"
    LUMEN = "lm"
    LUX = "lx"
    BECQUEREL = "Bq"
    GRAY = "Gy"
    SIEVERT = "Sv"
    KATAL = "kat"
    SQUARE_METER = "m2"
    CUBIC_METER = "m3"
    LITER = "l"
    VELOCITY = "m/s"
    ACCELERATION = "m/s2"
    CUBIC_METER_PER_SECOND = "m3/s"
    LITER_PER_SECOND = "l/s"
    WATT_PER_SQUARE_METER = "W/m2"
    CANDELA_PER_SQUARE_METER = "cd/m2"
    BIT = "bit"
    BIT_PER_SECOND = "bit/s"
    DEGREES_LATITUDE = "lat"
    DEGREES_LONGITUDE = "lon"
    PH = "pH"
    DECIBEL = "dB"
    DECIBEL_RELATIVE_TO_1_W = "dBW"
    BEL = "Bspl"
    COUNTER = "count"
    RATIO = "//"
    RELATIVE_HUMIDITY = "%RH"
    PERCENTAGE_REMAINING_BATTERY_LEVEL = "%EL"
    SECONDS_REMAINING_BATTERY_LEVEL = "EL"
    EVENT_RATE_PER_SECOND = "1/s"
    EVENT_RATE_PER_MINUTE = "1/min"
    BPM = "beat/min"
    BEATS = "beats"
    SIEMENS_PER_METER = "S/m"
    PERCENTAGE = "%"
